#!/usr/bin/python
# -*- coding: utf-8 -*-
#This file is part of pydsl.
#
#pydsl is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#pydsl is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with pydsl.  If not, see <http://www.gnu.org/licenses/>.

"""Parser module"""
from pydsl.Lex import lexer_factory
from pydsl.Config import load

__author__ = "Nestor Arocha"
__copyright__ = "Copyright 2008-2013, Nestor Arocha"
__email__ = "nesaro@gmail.com"

import logging
LOG = logging.getLogger(__name__)


def terminal_symbol_reducer(symbol, word, production, fixed_start = False):
    """ Reduces a terminal symbol """
    #FIXME: This is the same code than extract for alphabets, should be merged
    if fixed_start:
        max_begin = 1
    else:
        max_begin = len(word)
    validresults = []
    for begin in range(0, max_begin):
        for end in range(begin, len(word)+1):
            if symbol.check(word[begin:end]):
                validresults.append((end-begin, begin, end))
    validresults = sorted(validresults, key=lambda x:x[0])
    if symbol.boundariesrules == "min":
        validresults = validresults[:1]
    elif symbol.boundariesrules == "max":
        validresults = validresults[-1:]
    elif symbol.boundariesrules == "any":
        pass
    elif isinstance(symbol.boundariesrules , int):
        #validresults = [x for x in validresults if x[0] == symbol.boundariesrules]
        pass
    else:
        raise ValueError("Unknown boundaries rules")
    from pydsl.Tree import ParseTree
    return [ParseTree(begin, end, [symbol], word[begin:end], production) for (size, begin, end) in validresults]

class Parser(object):
    """Expands an input based on grammar rules
    At this time, all parsers are tree based"""
    def __init__(self, bnfgrammar):
        self._productionset = bnfgrammar

    def get_trees(self, word): # -> list:
        """ returns a ParseTree list with all guesses """
        raise NotImplementedError

    def __call__(self, word):
        return self.get_trees(word)

    @property
    def productionset(self):
        """returns productionset"""
        return self._productionset

class TopDownParser(Parser):
    """Top down parser like descent parser"""
    def __init__(self, bnfgrammar):
        Parser.__init__(self, bnfgrammar)

class BottomUpParser(Parser):
    """ leaf to root parser"""
    def __init__(self, bnfgrammar):
        self._lexer = lexer_factory(bnfgrammar.alphabet())
        Parser.__init__(self, bnfgrammar)


def parser_factory(grammar, parser = "auto"):
    if isinstance(grammar, str):
        grammar = load(grammar)
    from pydsl.Grammar.BNF import BNFGrammar
    if isinstance(grammar, BNFGrammar):
        if parser == "descent":
            from pydsl.Parser.RecursiveDescent import BacktracingErrorRecursiveDescentParser
            return BacktracingErrorRecursiveDescentParser(grammar)
        elif parser in ("auto" , "default" , "weighted"):
            #TODO Guess best parser
            from pydsl.Parser.Weighted import WeightedParser
            return WeightedParser(grammar)
        else:
            raise Exception("Wrong parser name: " + parser)
    else:
        raise ValueError(grammar)


def parse(definition, data, parser = "auto"):
    return parser_factory(definition, parser)(data)


#!/usr/bin/python
# -*- coding: utf-8 -*-
#This file is part of pydsl.
#
#pydsl is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#pydsl is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with pydsl.  If not, see <http://www.gnu.org/licenses/>.

__author__ = "Nestor Arocha"
__copyright__ = "Copyright 2008-2013, Nestor Arocha"
__email__ = "nesaro@gmail.com"

import logging
LOG = logging.getLogger(__name__)
from pydsl.Alphabet.Definition import AlphabetDefinition
from pydsl.Grammar.Definition import GrammarDefinition
from pydsl.Check import checker_factory

def extract(grammar, inputdata):
    """Extract every slice of the input data that belongs to the Grammar Definition"""
    checker = checker_factory(grammar)
    totallen = len(inputdata)
    if isinstance(grammar, GrammarDefinition):
        try:
            maxl = grammar.maxsize or totallen
        except NotImplementedError:
            maxl = totallen
        try:
            minl = grammar.minsize
        except NotImplementedError:
            minl = 1
    elif isinstance(grammar, AlphabetDefinition):
        maxl = totallen
        minl = 1
    else:
        raise TypeError
    maxwsize = maxl - minl + 1
    result = []
    for i in range(totallen):
        for j in range(i+minl, min(i+maxwsize+1, totallen+1)):
            check = checker.check(inputdata[i:j])
            if check:
                result.append((i,j, inputdata[i:j]))
    return result




#!/usr/bin/python
# -*- coding: utf-8 -*-
# This file is part of pydsl.
#
# pydsl is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
# pydsl is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pydsl.  If not, see <http://www.gnu.org/licenses/>.

"""ListLibrary"""
from pypository.utils import ImmutableDict
import logging
LOG = logging.getLogger(__name__)
from pypository.List import ListStorage
from pydsl.Alphabet.Definition import Encoding


__author__ = "Nestor Arocha"
__copyright__ = "Copyright 2008-2013, Nestor Arocha"
__email__ = "nesaro@gmail.com"


class EncodingStorage(ListStorage):

    @staticmethod
    def _generatekey(element):
        return element

    @staticmethod
    def load(index):
        return Encoding(index)

    def generate_all_summaries(self):
        return tuple([ImmutableDict({"identifier": x, "value": x, "iclass": "Encoding"}) for x in self._content])



#!/usr/bin/python
# -*- coding: utf-8 -*-
# This file is part of pydsl.
#
# pydsl is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
# pydsl is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pydsl.  If not, see <http://www.gnu.org/licenses/>.

"""Base Lexer classes"""
__author__ = "Nestor Arocha"
__copyright__ = "Copyright 2008-2013, Nestor Arocha"
__email__ = "nesaro@gmail.com"

from pydsl.Alphabet.Definition import Encoding
from pydsl.Check import checker_factory
from pydsl.Config import load



class Lexer(object):

    """Translates an input written in one Alphabet into another Alphabet"""
    @property
    def input_alphabet(self):
        raise NotImplementedError

    @property
    def output_alphabet(self):
        raise NotImplementedError


class EncodingLexer(Lexer):

    """Special Lexer that encodes from a string a reads a string"""

    def __init__(self, encoding):
        self.encoding = encoding

    def __call__(self, string):
        for x in string:
            yield x

    def lexer_generator(self, target):
        next(target)
        buffer = ""
        while True:
            element = (yield)
            buffer += element  # Asumes string
            for x in buffer:
                target.send(x)


class AlphabetLexer(Lexer):

    """Lexer receives an Alphabet in the initialization (A1).
    Receives an input that belongs to A1 and generates a list of tokens in a different Alphabet A2
    It is always described with a regular grammar"""

    def __init__(self):
        self.load(None)

    @property
    def current(self):
        """Returns the element under the cursor"""
        try:
            return self.string[self.index]
        except IndexError:
            return None

    def load(self, string):
        self.string = string
        self.index = 0

    def consume(self):
        self.index += 1

    def match(self, char):
        if self.current != char:
            raise Exception("%s doesn't match %s" % (self.current, char))
        self.consume()

    def nextToken(self):
        raise NotImplementedError

    def __call__(self, string):  # -> "TokenList":
        """Tokenizes input, generating a list of tokens"""
        self.string = string
        result = [x for x in self.nextToken()]
        return result

    def lexer_generator(self):
        """generator version of the lexer, yields a new token as soon as possible"""
        raise NotImplementedError


class AlphabetListLexer(AlphabetLexer):

    def __init__(self, alphabet):
        AlphabetLexer.__init__(self)
        self.alphabet = alphabet

    @property
    def current(self):
        """Returns the element under the cursor until the end of the string"""
        return self.string[self.index:]

    def nextToken(self):
        from pydsl.Tree import Sequence
        tree = Sequence() #This is the extract algorightm
        valid_alternatives = []
        for gd in self.alphabet.grammarlist:
            checker = checker_factory(gd)
            for left in range(0, len(self.string)):
                for right in range(left +1, len(self.string) +1 ):
                    if checker.check(self.string[left:right]):
                        valid_alternatives.append((left, right, gd))
        if not valid_alternatives:
            raise Exception("Nothing consumed")
        for left, right, gd in valid_alternatives:
            string = self.string[left:right]
            tree.append(left, right, string, check_position=False)

        right_length_seq = []
        for x in tree.generate_valid_sequences():
            if x[-1]['right'] == len(self.string):
                right_length_seq.append(x)
        for y in sorted(right_length_seq, key=lambda x:len(x))[0]: #Always gets the match with less tokens
            yield y['content']

    def lexer_generator(self, target):
        next(target)
        buffer = ""
        while True:
            element = (yield)
            buffer += element  # Asumes string
            for x in range(1, len(buffer)):
                currentstr = buffer[:x]
                for gd in self.alphabet.grammarlist:
                    checker = checker_factory(gd)
                    if checker.check(currentstr):
                        buffer = buffer[x:]
                        target.send(currentstr)


class ConceptLexer(Lexer):

    """Translates a set of concepts that belong to a ConceptAlphabet into another ConceptAlphabet"""

    def __init__(self, function):
        self._function = function

    def __call__(self, *args, **kwargs):
        result = self._function(*args, **kwargs)
        return result


def lexer_factory(alphabet):
    from pydsl.Alphabet.Definition import AlphabetListDefinition
    if isinstance(alphabet, str):
        alphabet = load(alphabet)
    if isinstance(alphabet, AlphabetListDefinition):
        return AlphabetListLexer(alphabet)
    elif isinstance(alphabet, Encoding):
        return EncodingLexer(alphabet)
    else:
        raise ValueError(alphabet)

def lex(definition, data):
    return lexer_factory(definition)(data)



#!/usr/bin/python
# -*- coding: utf-8 -*-
#This file is part of pydsl.
#
#pydsl is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#pydsl is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with pydsl.  If not, see <http://www.gnu.org/licenses/>.


__author__ = "Nestor Arocha"
__copyright__ = "Copyright 2008-2013, Nestor Arocha"
__email__ = "nesaro@gmail.com"

import logging
LOG = logging.getLogger(__name__)
from pydsl.Config import load
from pydsl.Parser.Parser import parser_factory


class Validator(object):
    """Receives a grammar and an input that doesn't belong to the grammar, 
    expands the grammar by parsing it, returns a list of what's needed 
    to become grammar compatible"""
    def __init__(self, grammar):
        self.gd = grammar

    def __call__(self, inputstring): #-> set
        raise NotImplementedError


class BNFValidator(Validator):
    def __call__(self, inputstring):
        parser = parser_factory(self.gd, "descent")
        resulttrees = parser.get_trees(inputstring, True)
        return [x for x in resulttrees if not x.valid]


def validator_factory(grammar):
    if isinstance(grammar, str):
        grammar = load(grammar)
    from pydsl.Grammar.BNF import BNFGrammar
    if isinstance(grammar, BNFGrammar):
        return BNFValidator(grammar)
    else:
        raise ValueError(grammar)


def validate(definition, data):
    return validator_factory(definition)(data)


#!/usr/bin/python
# -*- coding: utf-8 -*-
# This file is part of pydsl.
#
# pydsl is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
# pydsl is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pydsl.  If not, see <http://www.gnu.org/licenses/>.

"""Tree class for tree based parsers"""

__author__ = "Nestor Arocha"
__copyright__ = "Copyright 2008-2013, Nestor Arocha"
__email__ = "nesaro@gmail.com"

import logging
LOG = logging.getLogger(__name__)
from pydsl.Grammar.Symbol import TerminalSymbol


def traversePreOrder(item):
    result = [item]
    for child in item.childlist:
        result += traversePreOrder(child)
    return result


def traverseInOrder(item):
    result = [traverseInOrder(item.childlist[0]), item]
    for child in item.childlist[1:]:
        result += traverseInOrder(child)
    return result


def traversePostOrder(item):
    result = []
    for child in item.childlist:
        result += traversePostOrder(child)
    result.append(item)
    return result


class Tree(object):

    def __init__(self, childlist=None):
        if not childlist:
            childlist = []
        self.childlist = childlist

    def append_child(self, dpr):
        """appends dpr to childlist"""
        self.childlist.append(dpr)

    def to_list(self, order="preorder"):
        if order == "preorder":
            return traversePreOrder(self)
        elif order == "inorder":
            return traverseInOrder(self)
        elif order == "postorder":
            return traversePostOrder(self)
        else:
            raise ValueError("Unknown order %s" % order)

    def first_leaf(self):
        """Returns the first lead node"""
        if self.childlist:
            return self.childlist[0].first_leaf()
        else:
            return self


class PositionTree(Tree):

    """Stores the position of the original tree"""

    def __init__(self, leftpos, rightpos, content, production=None, valid=True, childlist=None):
        Tree.__init__(self, childlist)
        self.leftpos = leftpos
        self.rightpos = rightpos
        self.content = content
        self.valid = valid
        self.production = production

    def __eq__(self, other):
        try:
            return self.production == other.production and self.content == other.content and self.leftpos == other.leftpos and self.rightpos == other.rightpos and self.valid == other.valid
        except AttributeError:
            return False

    def __bool__(self):
        """checks if it is a null result"""
        return self.valid

    def __getitem__(self, key):
        result = []
        mylist = self.to_list()
        for element in mylist:
            if element.content == key:
                result.append(element)
        if not result:
            raise KeyError("Element not found %s" % key)
        return result

    def shift(self, amount):
        """ shifts position """
        if self.leftpos is not None:
            self.leftpos += amount
        if self.leftpos is not None:
            self.rightpos += amount

    def __len__(self):
        if self.rightpos is None and self.leftpos is None:
            return 0
        return self.rightpos - self.leftpos

    def coverage(self):
        if not self:
            return 0, len(self)
        if self.childlist:
            childtotal = 0
            childcoverage = 0
            for child in self.childlist:
                newcoverage, newtotal = child.coverage()
                childcoverage += newcoverage
                childtotal += newtotal
            assert(childtotal == len(self))
            return childcoverage, childtotal
        else:
            return len(self), len(self)

    def get_by_symbol(self, index):
        if isinstance(self.production, TerminalSymbol):
            # FIXME quick hack for terminal rules
            return [(self.leftpos, self.rightpos)]
        result = []
        if self.production.leftside[0].name == index:
            result.append((self.leftpos, self.rightpos))
        else:
            LOG.debug(
                "Not equal: " + str(self.production.leftside[0].name) + " and :" + str(index))
        for child in self.childlist:
            result += child.get_by_symbol(index)
        return result

    def __contains__(self, index):
        if isinstance(self.production, TerminalSymbol):
            # FIXME quick hack for terminal rules
            return index == self.production.name
        if not self.production.leftside:
            return False
        if self.production.leftside[0].name == index:
            return True
        for child in self.childlist:
            if child.get_by_symbol(index):
                return True
        return False


class ParseTree(PositionTree):

    """ Stores a descent parser iteration result """

    def __init__(self, leftpos, rightpos, symbollist, content, production, childlist=None, valid=True):
        if not isinstance(leftpos, int) and leftpos is not None:
            raise TypeError
        if not isinstance(rightpos, int) and rightpos is not None:
            raise TypeError
        if not isinstance(symbollist, list):
            raise TypeError
        from pydsl.Grammar.BNF import Production
        if production is not None and not (isinstance(production, (Production, TerminalSymbol))):
            raise TypeError(production)
        PositionTree.__init__(
            self, leftpos, rightpos, content, production, valid, childlist)
        self.symbollist = symbollist

    def __add__(self, other):
        """ Adds two results. Only if self.rightpos = other.leftpos and parents are the same """
        if not isinstance(other, ParseTree):
            raise TypeError
        if other == []:
            return ParseTree(self.leftpos, self.rightpos, self.symbollist,
                             self.content, self.production, self.childlist)  # FIXME: Must return a childlist copy
        if self.rightpos == other.leftpos and self.production == other.production:
            leftpos = self.leftpos
            rightpos = other.rightpos
            production = self.production
            content = self.content + other.content
            symbollist = self.symbollist + other.symbollist
            childlist = self.childlist + other.childlist
            return ParseTree(leftpos, rightpos, symbollist, content, production, childlist)
        else:
            LOG.warning("Unable to add parser results")
            raise Exception


class Sequence:
    def __init__(self):
        self.possible_items = []

    @property
    def current_right(self):
        if not self.possible_items:
            return set([0])
        return set(x['right'] for x in self.possible_items)

    def append(self, left, right, content, check_position=True):
        if left > right:
            raise Exception
        if check_position == True and left:
            if left not in self.current_right:
                raise ValueError("Unable to add element")
        self.possible_items.append({'left':left, 'right':right, 'content':content})

    def generate_valid_sequences(self):
        """Returns list"""
        valid_sets = [[x] for x in self.possible_items if x['left'] == 0]
        change = True
        while change:
            change = False
            for possible in self.possible_items:
                for current_valid in valid_sets[:]:
                    if possible['left'] == current_valid[-1]['right']:
                        if current_valid + [possible] not in valid_sets:
                            valid_sets.append(current_valid + [possible])
                            change = True
        return valid_sets







#!/usr/bin/python
# -*- coding: utf-8 -*-

#Copyright (C) 2008-2013 Nestor Arocha
#pydsl is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#pydsl is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with pydsl.  If not, see <http://www.gnu.org/licenses/>.

"""WeightedParser"""

import logging
LOG = logging.getLogger(__name__)
from .Parser import TopDownParser, terminal_symbol_reducer
from pydsl.Tree import ParseTree


def _create_combined_list(input_list):
    """
    Creates a list of lists, each sublist contains a permutation of inputs. E.g:
    [a, b, c] -> [[a1,b1,c1],[a2,b1,c1]] ....
    """

    midlist = [] #All blocks combinations are stored here. Each element is a list of viable sequences
    input_list = [x for x in input_list if x]
    if not input_list:
        return []
    validsets = 1

    #Processing head set

    for result in input_list[0]:
        if not(isinstance(result, ParseTree)):
            raise TypeError
        if not result.leftpos:
            midlist.append([result])

    #Processing Tail sets
    for resultl in input_list[1:]:
        #For each result list
        if not resultl:
            continue
        for result in resultl:
            #for each result
            for middleresult in midlist[:]:
                #Here we mix every result with intermediate results list
                prevright = [x.rightpos for x in middleresult if x.rightpos is not None][-1]
                if result.leftpos is None or prevright == result.leftpos:
                    midlist.append(middleresult + [result])
        validsets += 1

        #Removes all results that have less elements than the number of valid sets
        for element in midlist[:]:
            if len(element) != validsets:
                midlist.remove(element)

    return midlist

def mix_results(resultll, productionset):
    """ Mix n sets of results """
    #We mix all results into final result
    midlist = _create_combined_list(resultll)
    finallist = []
    for combination in midlist:
        if len(combination) == 1:
            finallist.append(combination[0])
        else:
            left_pos = [x.leftpos for x in combination if x.leftpos is not None][0]
            right_pos = [x.rightpos for x in combination if x.rightpos is not None][-1]
            #Creates a node with all elements, and originals nodes are the childs of the new node
            symbollist = []
            compoundword = ""
            for element in combination:
                compoundword += element.content
                symbollist += element.symbollist
            finalresult = ParseTree(left_pos, right_pos, symbollist, compoundword, combination[0].production, valid = all([x for x in combination]))
            #Add childs to result. FIXME Adding already created elements as children of the new one
            rightside = []
            for child in combination:
                assert(child != finalresult) #Avoid recursion
                finalresult.append_child(child)
                rightside += child.symbollist #Creating the rightside of the production to guess the full production #FIXME doesn't work with terminals
            try:
                finalresult.production = productionset.getProductionsBySide(rightside, "right")
            except IndexError:
                finalresult.production = None
            finally:
                finallist.append(finalresult) #rule found; we add bound together version
    return finallist

def locate_heavier_symbol(symbols):
    """ Locates the heavier symbol inside input list"""
    current_symbol = None
    for symbol in symbols:
        if current_symbol is None or symbol.weight > current_symbol.weight:
            current_symbol = symbol
    return current_symbol

class WeightedParser(TopDownParser):
    """Weighted Parser class"""
    def get_trees(self, data, showerrors = False):
        result = self.__recursive_parser(self._productionset.initialsymbol, data, self._productionset.main_production, showerrors)
        finalresult = []
        for eresult in result:
            if eresult.leftpos == 0 and eresult.rightpos == len(data):
                finalresult.append(eresult)
        return finalresult

    def __handle_alternative(self,symbols, data, production, showerrors):
        """Multiple symbols
        returns a list of possible reductions"""
        if len(symbols) == 1:
            return self.__recursive_parser(symbols[0], data, production, showerrors)
        heavy_symbol = locate_heavier_symbol(symbols)
        index = symbols.index(heavy_symbol)
        LOG.debug("Iteration: Call recursive heavy" + str(heavy_symbol) + str(index))
        heavyresult = self.__recursive_parser(heavy_symbol, data, production, showerrors)
        if not heavyresult:
            return []
        leftpos = max([x.leftpos for x in heavyresult])
        rightpos = min([x.rightpos for x in heavyresult]) or 0
        leftside = []
        rightside = []
        if symbols[:index]:
            LOG.debug("Iteration: Call recursive left")
            leftside = self.__handle_alternative(symbols[:index], data[:leftpos], production, showerrors)
        if symbols[(index+1):]:
            LOG.debug("Iteration: Call recursive right")
            rightside = self.__handle_alternative(symbols[(index+1):], data[rightpos:], production, showerrors)
            #shift right results
        for x in rightside:
            x.shift(rightpos)
        if not leftside and not rightside:
            return heavyresult
        result = mix_results([leftside, heavyresult, rightside], self._productionset)
        return result

    def __recursive_parser(self, onlysymbol, data, production, showerrors = False):
        """ Main function. It is recursive """
        from pydsl.Grammar.Symbol import TerminalSymbol, NonTerminalSymbol, NullSymbol
        if isinstance(onlysymbol, TerminalSymbol):
            #Locate every occurrence of word and return a set of results. Follow boundariesrules
            LOG.debug("Iteration: terminalsymbol")
            result = terminal_symbol_reducer(onlysymbol, data, onlysymbol) #FIXME add information about production
            if showerrors and not result:
                return [ParseTree(0,len(data), [onlysymbol] , data, onlysymbol, valid = False)] #FIXME add information about production
            return result
        elif isinstance(onlysymbol, NonTerminalSymbol):
            result = []
            for alternative in self._productionset.getProductionsBySide([onlysymbol]):
                #result += self.__recursive_parser(alternative.rightside, data, alternative, showerrors)
                alternative_result = self.__handle_alternative(alternative.rightside, data, alternative, showerrors)
                for x in alternative_result:
                    if list(x.symbollist) == list(alternative.rightside): #Filters incomplete attempts
                        result.append(ParseTree(x.leftpos, x.rightpos, [onlysymbol], data[x.leftpos:x.rightpos], production, valid = True))
                    #TODO: Add child
            if showerrors and not result:
                return [ParseTree(0, len(data), [onlysymbol], data, production, valid = False)]
            LOG.debug("Result " + str(result))
            return result
        elif isinstance(onlysymbol, NullSymbol):
            return[ParseTree(None, None, [onlysymbol], "", production)]
        raise Exception



#!/usr/bin/python
# -*- coding: utf-8 -*-
# This file is part of pydsl.
#
# pydsl is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
# pydsl is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pydsl.  If not, see <http://www.gnu.org/licenses/>.

"""Dictionary based library"""

__author__ = "Nestor Arocha"
__copyright__ = "Copyright 2008-2013, Nestor Arocha"
__email__ = "nesaro@gmail.com"

import logging
LOG = logging.getLogger(__name__)
from pypository.Memory import Memory

class RegexpDictStorage(Memory):
    def __init__(self, dictionary):
        Memory.__init__(self)
        self._content = dictionary

    def generate_all_summaries(self):# -> list:
        result = []
        from pypository.utils import ImmutableDict
        for key in self._content:
            result.append(ImmutableDict({"identifier":key, "regexp":self._content[key]["regexp"], "iclass":"RegularExpression"}))
        return result

    def load(self, index, **kwargs):
        import re
        flags = 0
        if "flags" in self._content[index]:
            if "i" in self._content[index]["flags"]:
                flags |= re.I
        from pydsl.Grammar.Definition import RegularExpressionDefinition
        return RegularExpressionDefinition(self._content[index]["regexp"], flags)

    def __iter__(self):
        self.index = 0
        self.cache = []
        self.cache += self.generate_all_summaries()
        return self

    def next(self):
        try:
            result = self.cache[self.index]
        except IndexError:
            raise StopIteration
        self.index += 1
        return result

    def __contains__(self, index):
        return index in self._content



#!/usr/bin/python
# -*- coding: utf-8 -*-
#This file is part of pydsl.
#
#pydsl is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#pydsl is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with pydsl.  If not, see <http://www.gnu.org/licenses/>.


"""
guess which types are the input data. 
"""
from pydsl.Check import check

__author__ = "Nestor Arocha"
__copyright__ = "Copyright 2008-2013, Nestor Arocha"
__email__ = "nesaro@gmail.com"
#TODO: Add Alphabet support


import logging
LOG = logging.getLogger(__name__)
from pypository.search.Searcher import Searcher
from pydsl.Config import GLOBALCONFIG


class Guesser(object):
    """Returns every grammar and alphabet definition that matches the input"""
    def __init__(self, memorylist = None):
        if not memorylist:
            memorylist = GLOBALCONFIG.memorylist
        self.memorylist = memorylist
        self.searcher = Searcher([x.indexer for x in memorylist])

    def __call__(self, data): #-> set:
        result = set()
        for summary in self.searcher.search():
            try:
                for mem in self.memorylist:
                    if summary["identifier"] in mem:
                        name = summary["identifier"]
                        try:
                            typ = mem.load(name)
                        except:
                            LOG.warning("Error while loading memory %s" % name)
                            continue
                        break
                else:
                    continue # not found 
                if check(typ, data):
                    result.add(str(name))
            except (TypeError, ValueError):
                continue
        return result

def guess(data):
    return Guesser()(data)



#!/usr/bin/python
# -*- coding: utf-8 -*-
#This file is part of pydsl.
#
#pydsl is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#pydsl is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with pydsl.  If not, see <http://www.gnu.org/licenses/>.


""" File Library class """

import logging
LOG = logging.getLogger(__name__)
from pypository.utils import ImmutableDict, getFileTuple, load_module
import imp

def load_python_file(moduleobject):
    """ Try to create an indexable instance from a module"""
    if isinstance(moduleobject, str):
        moduleobject = load_module(moduleobject)
    if not hasattr(moduleobject, "iclass"):
        raise KeyError("Element" + str(moduleobject))
    iclass = getattr(moduleobject, "iclass")
    resultdic = {}
    mylist = list(filter(lambda x:x[:1] != "_" and x != "iclass", (dir(moduleobject))))
    for x in mylist:
        resultdic[x] = getattr(moduleobject, x)
    if iclass == "SymbolGrammar":
        from pydsl.Grammar.BNF import BNFGrammar
        return BNFGrammar(**resultdic)
    elif iclass == "PLY":
        from pydsl.Grammar.Definition import PLYGrammar
        return PLYGrammar(moduleobject)
    elif iclass == "MongoDict":
        from pydsl.Grammar.Definition import MongoGrammar
        return MongoGrammar(resultdic)
    elif iclass in ["PythonGrammar"]:
        from pydsl.Grammar.Definition import PythonGrammar
        return PythonGrammar(resultdic)
    elif iclass == "PythonTransformer":
        return resultdic
    elif iclass == "AlphabetList":
        from pydsl.Alphabet.Definition import AlphabetListDefinition
        return AlphabetListDefinition(**resultdic)
    elif iclass == "pyparsing":
        return resultdic['root_symbol']
    else:
        raise ValueError(str(moduleobject))


def summary_python_file(modulepath):
    (_, _, fileBaseName, ext) = getFileTuple(modulepath)
    moduleobject = imp.load_source(fileBaseName, modulepath)
    result = {"identifier":fileBaseName, "iclass":moduleobject.iclass, "filepath":modulepath}
    if hasattr(moduleobject, "title"):
        result["title"] =  ImmutableDict(moduleobject.title)
    if hasattr(moduleobject, "description"):
        result["description"] =  ImmutableDict(moduleobject.description)
    if hasattr(moduleobject, "inputdic"):
        result["input"] = ImmutableDict(moduleobject.inputdic)
        result["inputlist"] = tuple(moduleobject.inputdic.values())
    if hasattr(moduleobject, "outputdic"):
        result["output"] = ImmutableDict(moduleobject.outputdic)
        result["outputlist"] = tuple(moduleobject.outputdic.values())
    if hasattr(moduleobject, "inputformat"):
        result["input"] = moduleobject.inputformat
    if hasattr(moduleobject, "outputformat"):
        result["output"] = moduleobject.outputformat
    if hasattr(moduleobject, "name"):
        result["name"] = moduleobject.name
    return ImmutableDict(result)




#!/usr/bin/python
# -*- coding: utf-8 -*-
#This file is part of pydsl.
#
#pydsl is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#pydsl is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with pydsl.  If not, see <http://www.gnu.org/licenses/>.

"""Recursive descent parser"""
from pydsl import Tree
from pydsl.Check import check

__author__ = "Nestor Arocha"
__copyright__ = "Copyright 2008-2013, Nestor Arocha"
__email__ = "nesaro@gmail.com"

import logging
LOG = logging.getLogger(__name__)
from .Parser import TopDownParser, terminal_symbol_reducer
from pydsl.Tree import ParseTree, Tree

#Errors are stored elsewhere. If another alternative covers this error, we discard this alternative.
#If not, we choose the alternative which covers more amount of input

class RecursiveDescentResultTree(Tree):
    """Stores alternatives generated by the parser. Internal use only"""
    def __init__(self, content):
        Tree.__init__(self)
        if content is not None and not isinstance(content, ParseTree):
            raise TypeError
        self.content = content

    def get_lists(self):
        """Converts the tree into a list of lists"""
        result = []
        mylist = [self.content]
        if not self.childlist and self.content:
            return [mylist]
        for child in self.childlist:
            x = child.get_lists()
            for li in x:
                assert(isinstance(li, list))
                if self.content is None:
                    result.append(li)
                elif self.content.rightpos == li[0].leftpos or self.content.rightpos == self.content.leftpos:
                    result.append(mylist + li)
        return result

    def append(self, content, startpos):
        if not isinstance(content, ParseTree):
            raise TypeError
        if self.content is None and startpos == 0:
            self.childlist.append(RecursiveDescentResultTree(content))
            return True
        elif not self.childlist and self.content.rightpos == startpos:
            self.childlist.append(RecursiveDescentResultTree(content))
            return True
        elif content.rightpos == content.leftpos and self.content is not None and self.content.rightpos == startpos:
            self.childlist.append(RecursiveDescentResultTree(content))
            return True
        elif self.content is not None and self.childlist and startpos == self.content.rightpos and content.rightpos != content.leftpos:
            self.childlist.append(RecursiveDescentResultTree(content))
            return True
        elif self.childlist:
            for x in self.childlist:
                if x.append(content, startpos):
                    return True
            return False
        elif content == self.content:
            LOG.debug("duplicated content")
            return False
        else:
            #TODO:Raise an exception? 
            return False

    def right_limit_list(self):
        """Returns a list with every right limit (inorder)"""
        result = []
        if self.content and self.content.rightpos:
            result.append(self.content.rightpos)
        for x in self.childlist:
            result += x.right_limit_list()
        if not result:
            result = [0]
        return list(set(result))


class BacktracingErrorRecursiveDescentParser(TopDownParser):
    """Recursive descent parser implementation. Backtracing. Null support. Error support"""
    def get_trees(self, data, showerrors = False): # -> list:
        """ returns a list of trees with valid guesses """
        result = self.__recursive_parser(self._productionset.initialsymbol, data, self._productionset.main_production, showerrors)
        finalresult = []
        for eresult in result:
            if eresult.leftpos == 0 and eresult.rightpos == len(data) and eresult not in finalresult:
                finalresult.append(eresult)        
        return finalresult

    def __recursive_parser(self, onlysymbol, data, production, showerrors = False):
        """ Aux function. helps check_word"""
        LOG.debug("__recursive_parser: Begin ")
        if not data:
            return []
        from pydsl.Grammar.Symbol import TerminalSymbol, NullSymbol, NonTerminalSymbol
        if isinstance(onlysymbol, TerminalSymbol):
            #Locate every occurrence of word and return a set of results. Follow boundariesrules
            LOG.debug("Iteration: terminalsymbol")
            result = terminal_symbol_reducer(onlysymbol, data, production, fixed_start=True)
            if showerrors and not result:
                return [ParseTree(0,len(data), [onlysymbol] , data, onlysymbol, valid = False)]
            return result
        elif isinstance(onlysymbol, NullSymbol):
            return [ParseTree(0, 0, [onlysymbol], "", production)]
        elif isinstance(onlysymbol, NonTerminalSymbol):
            validstack = []
            invalidstack = []
            for alternative in self._productionset.getProductionsBySide([onlysymbol]): #Alternative
                alternativetree = RecursiveDescentResultTree(None)
                alternativeinvalidstack = []
                for symbol in alternative.rightside: # Symbol
                    symbol_success = False
                    for totalpos in alternativetree.right_limit_list(): # Right limit
                        if totalpos >= len(data):
                            continue
                        thisresult =  self.__recursive_parser(symbol, data[totalpos:], alternative, showerrors)
                        if thisresult and all(thisresult):
                            symbol_success = True
                            for x in thisresult:
                                x.shift(totalpos)
                                success = alternativetree.append(x, totalpos)
                                if not success:
                                    #TODO: Add as an error to the tree or to another place
                                    LOG.debug("Discarded symbol :" + str(symbol) + " position:" + str(totalpos))
                                else:
                                    LOG.debug("Added symbol :" + str(symbol) + " position:" + str(totalpos))
                        else:
                            alternativeinvalidstack += [x for x in thisresult if not x]

                    if not symbol_success:
                        LOG.debug("Symbol doesn't work" + str(symbol))
                        break #Try next alternative
                else: # Alternative success (no break happened)
                    invalidstack += alternativeinvalidstack
                for x in alternativetree.get_lists():
                    validstack.append(x)
            result = []

            LOG.debug("iteration result collection finished:" + str(validstack))
            for alternative in self._productionset.getProductionsBySide([onlysymbol]):
                nullcount = alternative.rightside.count(NullSymbol())
                for results in validstack:
                    nnullresults = 0
                    leftpos = results[0].leftpos
                    rightpos = results[-1].rightpos
                    for y in [x.symbollist for x in results]:
                        nnullresults += y.count(NullSymbol())
                    if len(results) - nnullresults != len(alternative.rightside) - nullcount:
                        LOG.debug("Discarded: incorrect number of non null symbols")
                        continue
                    if rightpos > len(data):
                        LOG.debug("Discarded: length mismatch")
                        continue
                    for x in range(min(len(alternative.rightside), len(results))):
                        if results[x].content != alternative.rightside[x]:
                            LOG.debug("Discarded: rule doesn't match partial result")
                            continue
                    childlist = [x for x in results]
                    allvalid = all([x.valid for x in childlist])
                    if allvalid:
                        newresult = ParseTree(0, rightpos - leftpos, [onlysymbol],
                                data[leftpos:rightpos], alternative, childlist)
                        newresult.valid = True
                        result.append(newresult)
            if showerrors and not result:
                erroresult = ParseTree(0,len(data), [onlysymbol] , data, production, valid = False)
                for invalid in invalidstack:
                    current_symbol = invalid.production if isinstance(invalid.production, (TerminalSymbol, NullSymbol)) else invalid.production.leftside[0]
                    if current_symbol in production.rightside:
                        erroresult.append_child(invalid)
                return [erroresult]
            return result
        raise Exception("Unknown symbol:" + str(onlysymbol))

class BacktracingRecursiveDescentParser(TopDownParser):
    """Recursive descent parser implementation. Backtracing"""
    def get_trees(self, data, showerrors = False): # -> list:
        """ returns a list of trees with valid guesses """
        if showerrors:
            raise NotImplementedError("This parser doesn't implement errors")
        result = self.__recursive_parser(self._productionset.initialsymbol, data, self._productionset.main_production)
        finalresult = []
        for eresult in result:
            if eresult.leftpos == 0 and eresult.rightpos == len(data) and eresult not in finalresult:
                finalresult.append(eresult)        
        return finalresult

    def __recursive_parser(self, onlysymbol, data, production):
        """ Aux function. helps check_word"""
        LOG.debug("__recursive_parser: Begin ")
        if not data:
            return []
        from pydsl.Grammar.Symbol import TerminalSymbol, NonTerminalSymbol
        if isinstance(onlysymbol, TerminalSymbol):
            #Locate every occurrence of word and return a set of results. Follow boundariesrules
            LOG.debug("Iteration: terminalsymbol")
            result = terminal_symbol_reducer(onlysymbol, data, production, fixed_start=True)
            return result
        elif isinstance(onlysymbol, NonTerminalSymbol):
            validstack = []
            for alternative in self._productionset.getProductionsBySide([onlysymbol]): #Alternative
                alternativetree = RecursiveDescentResultTree(None)
                for symbol in alternative.rightside: # Symbol
                    symbol_success = False
                    for totalpos in alternativetree.right_limit_list(): # Right limit
                        if totalpos >= len(data):
                            continue
                        thisresult =  self.__recursive_parser(symbol, data[totalpos:], alternative)
                        if thisresult and all(thisresult):
                            symbol_success = True
                            for x in thisresult:
                                x.shift(totalpos)
                                success = alternativetree.append(x, totalpos)
                    if not symbol_success:
                        LOG.debug("Symbol doesn't work" + str(symbol))
                        break #Try next alternative
                for x in alternativetree.get_lists():
                    validstack.append(x)
            result = []

            LOG.debug("iteration result collection finished:" + str(validstack))
            for alternative in self._productionset.getProductionsBySide([onlysymbol]):
                for results in validstack:
                    leftpos = results[0].leftpos
                    rightpos = results[-1].rightpos
                    if len(results) != len(alternative.rightside): 
                        LOG.debug("Discarded: incorrect number of non null symbols")
                        continue
                    if rightpos > len(data):
                        LOG.debug("Discarded: length mismatch")
                        continue
                    for x in range(min(len(alternative.rightside), len(results))):
                        if results[x].content != alternative.rightside[x]:
                            LOG.debug("Discarded: rule doesn't match partial result")
                            continue
                    childlist = [x for x in results]
                    newresult = ParseTree(0, rightpos - leftpos, [onlysymbol],
                            data[leftpos:rightpos], alternative, childlist)
                    newresult.valid = True
                    result.append(newresult)
            return result
        raise Exception("Unknown symbol:" + str(onlysymbol))

class LL1RecursiveDescentParser(TopDownParser):
    def get_trees(self, data, showerrors = False): # -> list:
        """ returns a list of trees with valid guesses """
        if showerrors:
            raise NotImplementedError("This parser doesn't implement errors")
        self.data = data
        self.index = 0
        return [self.__aux_parser(self._productionset.initialsymbol)]

    def __aux_parser(self, symbol):
        from pydsl.Grammar.Symbol import TerminalSymbol
        if isinstance(symbol, TerminalSymbol):
            LOG.debug("matching symbol %s, data:%s, index:%s" % (symbol,self.data,self.index ))
            result= self.match(symbol)
            LOG.debug("symbol matched %s" % result)
            return result
        productions = self._productionset.getProductionsBySide(symbol)
        first_of_each_production = {}
        for production in productions:
            first_of_each_production[production] = [x.gd for x in self._productionset.first_lookup(production.rightside[0])]
        valid_firsts = [production_instance for (production_instance,first_set) in first_of_each_production.items() if check(first_set, self.current)]
        if len(valid_firsts) != 1:
            raise Exception("Expected only one valid production, found %s" % len(valid_firsts))
        childlist = []
        for element in valid_firsts[0].rightside:
            childlist.append(self.__aux_parser(element))
        left = childlist[0].leftpos
        right = childlist[-1].rightpos
        content = [x.content for x in childlist]
        return ParseTree(left, right, [symbol], content, valid_firsts[0], childlist=childlist)


    def consume(self):
        self.index +=1
        if self.index > len(self.data):
            raise IndexError("Attempted to consume index %s of data %s" % (self.index, self.data))

    @property
    def current(self):
        result = self.data[self.index]
        return result

    def match(self, symbol):
        if symbol.check(self.current):
            current = self.current
            self.consume()
            return ParseTree(self.index-1, self.index, [symbol], current, None)
        else:
            raise Exception("Not matched")





#!/usr/bin/python
# -*- coding: utf-8 -*-
# This file is part of pydsl.
#
# pydsl is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
# pydsl is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pydsl.  If not, see <http://www.gnu.org/licenses/>.

__author__ = "Nestor Arocha"
__copyright__ = "Copyright 2008-2013, Nestor Arocha"
__email__ = "nesaro@gmail.com"


class GrammarDefinition(object):

    def __init__(self):
        pass

    def enum(self):
        """Generates every possible accepted string"""
        raise NotImplementedError

    @property
    def first(self):# -> set:
        """List of possible first elements"""
        return [x for x in self.alphabet().to_list]

    @property
    def minsize(self):# -> int:
        """Returns the minimum size in alphabet tokens"""
        return 0

    @property
    def maxsize(self):
        """Returns the max size in alphabet tokens"""
        return None

    def alphabet(self):
        """Returns the alphabet used by this grammar"""
        raise NotImplementedError

class PLYGrammar(GrammarDefinition):
    """PLY based grammar"""
    def __init__(self, module):
        GrammarDefinition.__init__(self)
        self.module = module

    @property
    def maxsize(self):
        raise NotImplementedError

    @property
    def minsize(self):
        raise NotImplementedError

class RegularExpressionDefinition(GrammarDefinition):
    def __init__(self, regexp, flags = 0):
        if not isinstance(regexp, str):
            raise TypeError
        GrammarDefinition.__init__(self)
        self.regexpstr = regexp
        self.flags = flags
        import re
        self.regexp = re.compile(regexp, flags)

    def __hash__(self):
        return hash(self.regexpstr)

    def __eq__(self, other):
        if not isinstance(other, RegularExpressionDefinition):
            return False
        return self.regexpstr == other.regexpstr and self.flags == other.flags

    def __str__(self):
        return self.regexpstr

    @property
    def first(self):# -> set:
        i = 0
        while True:
            if self.regexpstr[i] == "^":
                i+=1
                continue
            if self.regexpstr[i] == "[":
                return [StringGrammarDefinition(x) for x in self.regexpstr[i+1:self.regexpstr.find("]")]]
            return [StringGrammarDefinition(self.regexpstr[i])]

    def __getattr__(self, attr):
        return getattr(self.regexp, attr)

    def alphabet(self):
        from pydsl.Alphabet.Definition import Encoding
        return Encoding("ascii")

class StringGrammarDefinition(GrammarDefinition):
    def __init__(self, string):
        GrammarDefinition.__init__(self)
        self.string = string

    def __hash__(self):
        return hash(self.string)

    def __eq__(self, other):
        try:
            return self.string == other.string
        except AttributeError:
            return False

    @property
    def first(self):
        return [StringGrammarDefinition(self.string[0])]

    def enum(self):
        yield self.string

    @property
    def maxsize(self):
        return len(self.string)

    @property
    def minsize(self):
        return len(self.string)

    def __str__(self):
        return str(self.string)

    def alphabet(self):
        return [StringGrammarDefinition(x) for x in self.string]

class JsonSchema(GrammarDefinition, dict):
    def __init__(self, *args, **kwargs):
        GrammarDefinition.__init__(self)
        dict.__init__(self, *args, **kwargs)

    def alphabet(self):
        from pydsl.Alphabet.Definition import Encoding
        return Encoding("ascii")

class MongoGrammar(GrammarDefinition, dict):
    def __init__(self, *args, **kwargs):
        GrammarDefinition.__init__(self)
        dict.__init__(self, *args, **kwargs)

    @property
    def first(self):
        return [StringGrammarDefinition("{")]

    def alphabet(self):
        from pydsl.Alphabet.Definition import Encoding
        return Encoding("ascii")

class PythonGrammar(GrammarDefinition, dict):
    def __init__(self, *args, **kwargs):
        GrammarDefinition.__init__(self)
        dict.__init__(self, *args, **kwargs)

    def alphabet(self):
        if "alphabet" in self:
            return self['alphabet']
        from pydsl.Alphabet.Definition import Encoding
        return Encoding("ascii")



#!/usr/bin/python
# -*- coding: utf-8 -*-
#This file is part of pydsl.
#
#pydsl is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#pydsl is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with pydsl.  If not, see <http://www.gnu.org/licenses/>.

__author__ = "Nestor Arocha"
__copyright__ = "Copyright 2008-2013, Nestor Arocha"
__email__ = "nesaro@gmail.com"


class AlphabetDefinition(object):
    """Defines a set of valid elements"""
    @property
    def to_list(self):
        """Returns a list of allowed grammars"""
        raise NotImplementedError

    def __contains__(self, token):
        """Returns true if the alphabet contains the token"""
        raise NotImplementedError

class AlphabetListDefinition(AlphabetDefinition):
    """Uses a list of grammar definitions"""
    def __init__(self, grammarlist):
        if not grammarlist:
            raise ValueError
        from pydsl.Config import load
        self.grammarlist = []
        for x in grammarlist:
            if isinstance(x, str):
                self.grammarlist.append(load(x))
            else:
                self.grammarlist.append(x)
        from pydsl.Grammar.Definition import GrammarDefinition
        for x in self.grammarlist:
            if not isinstance(x, GrammarDefinition):
                raise TypeError("Expected GrammarDefinition, Got %s:%s" % (x.__class__.__name__,x))

    def __getitem__(self, index):
        """Retrieves token by index"""
        return self.grammarlist[index]

    @property
    def to_list(self):
        return self.grammarlist


class Encoding(AlphabetDefinition):
    """Defines an alphabet using an encoding string"""
    def __init__(self, encoding):
        self.encoding = encoding

    def __getitem__(self, item):
        from pydsl.Check import EncodingChecker
        if EncodingChecker(self).check(item):
            from pydsl.Grammar.Definition import StringGrammarDefinition
            return StringGrammarDefinition(item)
        raise KeyError

    @property
    def to_list(self):
        #FIXME: Only ascii
        from pydsl.Grammar.Definition import StringGrammarDefinition
        return [StringGrammarDefinition(chr(x)) for x in range(128)]


class ConceptAlphabet(AlphabetDefinition):
    """Stores a list of concepts"""
    def __init__(self, conceptlist):
        self.conceptlist = conceptlist

    def __getitem__(self, item):
        return self.conceptlist[item]

    @property
    def to_list(self):
        return self.conceptlist



#!/usr/bin/python
# -*- coding: utf-8 -*-
# This file is part of pydsl.
#
# pydsl is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
# pydsl is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pydsl.  If not, see <http://www.gnu.org/licenses/>.

"""Global (per execution) elements"""

__author__ = "Nestor Arocha"
__copyright__ = "Copyright 2008-2013, Nestor Arocha"
__email__ = "nesaro@gmail.com"

import logging
LOG = logging.getLogger(__name__)
from pkg_resources import resource_filename


def load_default_memory():
    from pydsl.Memory.Dict import RegexpDictStorage
    from pydsl.Memory.List import EncodingStorage
    from pypository.Directory import DirStorage
    from regexps import res
    dirname = resource_filename("pydsl.contrib", "")
    GLOBALCONFIG.memorylist.append(
        DirStorage(
            dirname + "/grammar/",
            GLOBALCONFIG.formatlist))
    GLOBALCONFIG.memorylist.append(
        DirStorage(
            dirname + "/alphabet/",
            GLOBALCONFIG.formatlist))
    GLOBALCONFIG.memorylist.append(RegexpDictStorage(res))
    GLOBALCONFIG.memorylist.append(EncodingStorage(dirname + "/encoding.py"))
    GLOBALCONFIG.memorylist.append(
        DirStorage(
            dirname + "/transformer/",
            GLOBALCONFIG.formatlist))


def default_formats():
    from pydsl.Memory.File.Regexp import load_re_from_file, summary_re_from_file
    from pydsl.Memory.File.BNF import load_bnf_file, summary_bnf_file
    from pydsl.Memory.File.Python import summary_python_file, load_python_file
    return [
        {"extension": ".py",
         "summary_from_file": summary_python_file,
         "load_from_file": load_python_file},
        {"extension": ".re",
         "summary_from_file": summary_re_from_file,
         "load_from_file": load_re_from_file},
        {"extension": ".bnf",
         "summary_from_file": summary_bnf_file,
         "load_from_file": load_bnf_file},
    ]


class GlobalConfig(object):

    """Execution time global configuration"""

    memorylist = []
    def __init__(self, debuglevel=40):
        self.formatlist = default_formats()
        self.__debuglevel = debuglevel

    @property
    def debuglevel(self):
        return self.__debuglevel

    @debuglevel.setter
    def debuglevel(self, level):
        self.__debuglevel = level

    @classmethod
    def load(cls, identifier, memorylist=None):
        if not memorylist:
            memorylist = cls.memorylist
        from pypository.Loader import load
        return load(identifier, memorylist)

    @classmethod
    def search(cls, query, memorylist=None):
        if not memorylist:
            memorylist = cls.memorylist
        from pypository.Loader import search
        return search(query, memorylist)

class Singleton(type):

    """singleton pattern metaclass"""
    # Only problem here is that classes can't have two metaclasses
    def __init__(cls, name, bases, dct):
        cls.__instance = None
        type.__init__(cls, name, bases, dct)

    def __call__(cls, *args, **kw):
        if cls.__instance is None:
            cls.__instance = type.__call__(cls, *args, **kw)
        return cls.__instance

GlobalConfig2 = Singleton('GlobalConfig2', (GlobalConfig, ), {})
GLOBALCONFIG = GlobalConfig2()  # The only instance available

load = GLOBALCONFIG.load
search = GLOBALCONFIG.search



#!/usr/bin/python
# -*- coding: utf-8 -*-

#Copyright (C) 2008-2013 Nestor Arocha

"""IntegerTreeGrammar language"""

#{<int>\0parent\0<int>;}

def matchFun(input):
    parentrelationlist = str(input).split(";")
    from pydsl.Config import load
    integer = load("integer")
    for parentrelation in parentrelationlist:
        relation = parentrelation.split("\0")
        if len(relation) < 2:
            break
        if len(relation) != 3:
            return False
        if relation[1] != "parent":
            return False
        if not integer.check(relation[0]):
            return False
        if not integer.check(relation[2]):
            return False
    return True


iclass = "PythonGrammar"
auxdic =  {"int":"integer"}



#!/usr/bin/python
# -*- coding: utf-8 -*-
#This file is part of pydsl.
#
#pydsl is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#pydsl is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with pydsl.  If not, see <http://www.gnu.org/licenses/>.


__author__ = "Nestor Arocha"
__copyright__ = "Copyright 2008-2013, Nestor Arocha"
__email__ = "nesaro@gmail.com"

import logging
from collections import Iterable
LOG = logging.getLogger(__name__)


def check(definition, data):
    checker = checker_factory(definition)
    return checker(data)

def checker_factory(grammar):
    from pydsl.Grammar.BNF import BNFGrammar
    from pydsl.Grammar.Definition import PLYGrammar, RegularExpressionDefinition, MongoGrammar, StringGrammarDefinition, PythonGrammar
    from pydsl.Alphabet.Definition import AlphabetListDefinition, Encoding
    from collections import Iterable
    if isinstance(grammar, str):
        from pydsl.Config import load
        grammar = load(grammar)
    if isinstance(grammar, BNFGrammar):
        return BNFChecker(grammar)
    elif isinstance(grammar, RegularExpressionDefinition):
        return RegularExpressionChecker(grammar)
    elif isinstance(grammar, PythonGrammar) or isinstance(grammar, dict) and "matchFun" in grammar:
        return PythonChecker(grammar)
    elif isinstance(grammar, MongoGrammar):
        return MongoChecker(grammar["spec"])
    elif isinstance(grammar, PLYGrammar):
        return PLYChecker(grammar)
    elif isinstance(grammar, AlphabetListDefinition):
        return AlphabetListChecker(grammar)
    elif isinstance(grammar, StringGrammarDefinition):
        return StringChecker(grammar)
    elif isinstance(grammar, Encoding):
        return EncodingChecker(grammar)
    elif isinstance(grammar, Iterable):
        return IterableChecker(grammar)
    else:
        raise ValueError(grammar)


class Checker(object):
    """ Ensures that input follows a rule, protocol, grammar alphabet..."""
    def __init__(self):
        pass

    def __call__(self, value):
        return self.check(value)

    def check(self, value):# -> bool:
        raise NotImplementedError

class RegularExpressionChecker(Checker):
    def __init__(self, regexp, flags = ""):
        Checker.__init__(self)
        import re
        self.__regexpstr = regexp
        myflags = 0
        if "i" in flags:
            myflags |= re.I
        if isinstance(regexp, str):
            self.__regexp = re.compile(regexp, myflags)
        else:
            self.__regexp = regexp

    def check(self, data):
        """returns True if any match any regexp"""
        if isinstance(data, Iterable):
            data = "".join([str(x) for x in data])
        try:
            data = str(data)
        except UnicodeDecodeError:
            return False
        if not data:
            return False
        return bool(self.__regexp.match(data))


class BNFChecker(Checker):
    """Calls another program to perform checking. Args are always file names"""
    def __init__(self, bnf, parser = "auto"):
        Checker.__init__(self)
        parser = bnf.options.get("parser",parser)
        if parser == "descent" or parser == "auto" or parser == "default":
            from pydsl.Parser.RecursiveDescent import BacktracingErrorRecursiveDescentParser
            self.__parser = BacktracingErrorRecursiveDescentParser(bnf)
        elif parser == "weighted":
            from pydsl.Parser.Weighted import WeightedParser
            self.__parser = WeightedParser(bnf)
        else:
            LOG.error("Wrong parser name: " + parser)
            raise Exception

    def check(self, data):
        try:
            return len(self.__parser.get_trees(data)) > 0
        except IndexError:
            return False 

class PythonChecker(Checker):
    def __init__(self, module):
        Checker.__init__(self)
        self._matchFun = module["matchFun"]

    def check(self, data):
        try:
            return self._matchFun(data)
        except UnicodeDecodeError:
            return False


class MongoChecker(Checker):
    def __init__(self, dic):
        Checker.__init__(self)
        self.mongodic = dic

    def check(self, data):
        return self.__auxcheck(self.mongodic, data)

    def __auxcheck(self, specdict, data):
        """Recursive checker implementation"""
        for key, spec in specdict.items():
            value = data.get(key)
            if key == "$or" and len(specdict) == 1:
                return any([self.__auxcheck(x, data) for x in spec])
            elif isinstance(spec, dict) and len(spec) == 1:
                operator = list(spec.keys())[0]
                operand = list(spec.values())[0]
                if operator == "$type":
                    if not checker_factory(operand).check(str(value)):
                        return False
                elif operator == "$or":
                    if not any([self.__auxcheck({key:x}, data) for x in operand]):
                        return False
                else: #unknown operator
                    return spec == value
            elif isinstance(spec, dict):
                if not self.__auxcheck(spec, value):
                    return False
            else:
                if spec != value: 
                    return False
        return True

class PLYChecker(Checker):
    def __init__(self, gd):
        Checker.__init__(self)
        self.module = gd.module

    def check(self, data):
        if isinstance(data, Iterable):
            data = "".join([str(x) for x in data])
        from ply import yacc, lex
        lexer = lex.lex(self.module)
        parser = yacc.yacc(module = self.module)
        from pydsl.Exceptions import ParseError
        try:
            parser.parse(data, lexer = lexer)
        except ParseError:
            return False
        return True

class StringChecker(Checker):
    def __init__(self, gd):
        Checker.__init__(self)
        if isinstance(gd, str):
            from pydsl.Grammar.Definition import StringGrammarDefinition
            gd = StringGrammarDefinition(gd)
        self.gd = gd

    def check(self, data):
        if isinstance(data, Iterable):
            data = "".join([str(x) for x in data])
        return self.gd.string == str(data)

class JsonSchemaChecker(Checker):
    def __init__(self, gd):
        Checker.__init__(self)
        self.gd = gd

    def check(self, data):
        from jsonschema import validate, ValidationError
        try:
            validate(data, self.gd)
        except ValidationError:
            return False
        return True

class AlphabetListChecker(Checker):
    def __init__(self, gd):
        Checker.__init__(self)
        from pydsl.Alphabet.Definition import AlphabetListDefinition
        if not isinstance(gd, AlphabetListDefinition):
            raise TypeError
        self.gd = gd
        self.checkerinstances = [checker_factory(x) for x in self.gd.grammarlist]

    def check(self, data):
        if isinstance(data, str):
            data = [data]
        elif isinstance(data, Iterable):
            new_data = []
            for x in data:
                if isinstance(x, str):
                    new_data.append(x)
                else:
                    raise ValueError
            data = new_data
        for element in data:
            if not any((x.check(element) for x in self.checkerinstances)):
                return False
        return True

class EncodingChecker(Checker):
    def __init__(self, gd):
        Checker.__init__(self)
        self.gd = gd

    def check(self,data):
        encoding = self.gd.encoding
        if isinstance(data, Iterable):
            data = "".join([str(x) for x in data])
        if isinstance(data, str):
            try:
                data.encode(encoding)
            except UnicodeEncodeError:
                return False
            return True
        if isinstance(data, bytes):
            try:
                data.decode(encoding)
            except UnicodeDecodeError:
                return False
            return True
        return False

class IterableChecker(Checker):
    def __init__(self, iterable):
        Checker.__init__(self)
        self.iterable = iterable

    def check(self,data):
        for definition in self.iterable:
            try:
                if check(definition, data):
                    return True
            except KeyError:
                pass
        return False
